;*****************************************************************************/
;
;	PROJECT:	PSOC MORSE DECODER
; 	FILE NAME: 	MULTIPLY.INC
; 	AUTHOR:		Melchor Varela
;
; 	DESCRIPTION
;
;  	Multiply macros from AN2032 and AN2038
;	(c) Cypress Microsystems
;
; 	HISTORY
;
;	NAME   	DATE		REMARKS
;
; 	MVM		11OCT2003	Creation
;
;*****************************************************************************/
;*****************************************************************************/
;
;	$Workfile:   morsedecoder.asm  $
;	$Author:   mvm  $
;	$Date:   Oct 10 2003 23:47:18  $
;	$Revision:   1.2  $
;
;*****************************************************************************/

macro Multiply16s_8s_8s
;  @0 = @1 * @2
; mutliplies two 8 bit signed values and returns
;  a 16 bit signed value. 
   PushMulX @1                                         ;10 4
   PushMulY @2                                         ;10 4                               ;20
   GetLSB                                              ; 6 2
   mov [@0 + 1],A                                      ; 5 2
   GetSignedMSB                                        ; 6 2
   mov [@0],A                                          ; 5 2
endm                                                   ;42 total 16bytes
                                                  ;31 total 12bytes
;------------------------------------------------------------              
macro PushMulX
   mov A,[@0]                                          ; 5
   mov reg[MUL_X],A                                    ; 5 
endm                                                   ;10 total 4bytes
;-----------------------------------------------------------------------
macro PushMulY
   mov A,[@0]                                          ; 5
   mov reg[MUL_Y],A                                    ; 5 
endm                                                   ;10 total 4bytes 
;-----------------------------------------------------------------------
macro GetLSB
   mov A,reg[MUL_DL]                                   ; 6                           
endm                                                   ; 6 total 2bytes
;-----------------------------------------------------------------------
macro GetSignedMSB
   mov A,reg[MUL_DH]                                   ; 6
endm                                                   ; 6 total 2bytes
;-----------------------------------------------------------------------
macro GetUnsignedMSB
; @0 @1 & MUL_DH determine value
   mov A,reg[MUL_DH]                                   ; 6
   tst [@0],80h                                        ; 8
   jz .+4 ;(if msb is high)                            ; 5
      add A,[@1]                                       ; 6
  ;endif
   tst [@1],80h                                        ; 8
   jz .+4 ;if (if msb is high)                         ; 5
     add A,[@0]                                        ; 6
  ;endif2
endm                                                   ;44 total 16bytes
;--------------------------------------------------------------------- 

 
macro Multiply32s_16s_16s


;  (1)
   Multiply16s_8s_8s (@0 ),(@1 ),(@2)    	 ;42 16  
;  (2)
   PushMulY  (@2 + 1)                        ;10 4 
   GetXsYuMSB  (@1 ), (@2+1)   			     ;25 9
   cmp A,128                                 ; 8 2
   jc . + 4 ;pass on carry                   ; 5 2
      dec [@0]                               ; 7 2
   add [@0 + 1],A                            ; 7 2
   adc [@0],0                                ; 9 3
   GetLSB                                    ; 6 2
   mov [@0 +2],A                             ; 6 2
;  (3)
   PushMulX (@1 +1)                          ;10 4
   GetUnsignedMSB (@1+1),(@2 + 1)            ;25 9
   add [@0+2],A                              ; 7 2
   adc [@0+1],0                              ; 9 3
   adc [@0],0                                ; 9 3
   GetLSB                                    ; 6 2
   mov [@0+3],A                              ; 5 2
;  (4)
   PushMulY (@2)	    					 ;10 4 
   GetXuYsMSB (@1 + 1),(@2)                 ;25 9
   push A                                    ; 4 1  
   cmp A,128                                 ; 8 2
   jc .+ 4                                   ; 5 2
      dec [@0]                               ; 7 2
   GetLSB                                    ; 6 2
   add [@0 + 2],A                            ; 7 2
   pop A                                     ; 5 1
   adc [@0 +1],A                             ; 7 2
   adc [@0],0                                ; 9 3
  
endm                                         ;293 total 99bytes
;------------------------------------------------------------              
macro GetXuYsMSB
; @0 @1 & MUL_DH determine value
   mov A,reg[MUL_DH]                         ; 6  
   tst [@0],80h                              ; 8
   jz .+4 ;(if msb is high)                  ; 5
      add A,[@1]                             ; 6
  ;endif
endm                                         ;25 total 9bytes
;--------------------------------------------------------------------------
 macro GetXsYuMSB
; @0 @1 & MUL_DH determine value
   mov A,reg[MUL_DH]                         ; 6
   tst [@1],80h                              ; 8
   jz .+4 ;if (if msb is high)               ; 5
     add A,[@0]                              ; 6
  ;endif2
endm                                         ;25 total 9bytes
;--------------------------------------------------------------------------
